#!/bin/bash

# This script requires numdiff, diff, cut, wc, tail, tr, sed and grep
# All but numdiff are very standard UNIX utilities
# numdiff is available via Homebrew (brew install numdiff) as well as
# being available via most package managers (sudo apt-get install numdiff)

# "sed -i" is not used, due to different syntax on Linux vs MacOS

echo c2x test suite version 2.41b
echo

# Name of executable to test
c2x=${1:-c2x}

diff=diff
numdiff="numdiff"
tol1='-a 2e-6'
tol2='-r 1e-5'
tol="$tol1 $tol2"

passes=0
fails=0
total_passes=0
total_fails=0

fail(){
    echo Fail
    fails=$((fails+1))
}

bpass(){
  if [ -f $1 ] && [ -s $1 ] && cmp $1 $2 > /dev/null 2>&1
    then
	echo pass
	passes=$((passes+1))
    else
	echo Fail
	fails=$((fails+1))
    fi
}

pass(){
    if [ -f $1 ] && [ "`wc -l < $1`" -gt 0 ] && $diff -w $1 $2 > /dev/null 2>&1
    then
	echo pass
	passes=$((passes+1))
    else
	echo Fail
	fails=$((fails+1))
    fi
}

npass(){
    if [ -f $1 ] && [ "`wc -l < $1`" -gt 0 ] && $numdiff $tol $1 $2 > /dev/null 2>&1
    then
	echo pass
	passes=$((passes+1))
    else
	echo Fail
	fails=$((fails+1))
    fi
}

echo  "Testing: "
$c2x --version 2>&1 | head -3

# Test ability to expand XSF file with different prim and conv coords

echo -n 'XSF expansion: '
$c2x --cell SiC8.xsf > out.cell
pass out.cell SiC8.cell

# Test ability to expand CIF file with symmetry operations to expand

echo -n 'CIF expansion: '
$c2x --cell SiC8.cif | sed -e '/^%block SYMMETRY_OPS/,$d' | \
    sed -e '$d' > out.cell
pass out.cell SiC8.cell

echo -n 'CIF expansion (cartesians): '
$c2x --cell SiC8_cart.cif | sed -e '/^%block SYMMETRY_OPS/,$d' | \
    sed -e '$d' > out.cell
pass out.cell SiC8.cell

echo -n 'CIF reduction: '
$c2x -r --cif --sym SiC8.cell t.cif 2>/dev/null
grep -c Si t.cif > out1.dat
echo 1 > out2.dat
pass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'CIF reduction (2): '
$c2x --cell -n t.cif t.cell
npass t.cell SiC8.cell
rm t.cif t.cell

# Test ability to read pdb files

echo -n 'PDB read: '
$c2x --cell SiC8.pdb out.cell
pass out.cell SiC8.cell

echo -n 'PDBn read: '
$c2x --cell SiC8n.pdb out.cell
pass out.cell SiC8.cell

# Test ability to read SHELX97 files

echo -n 'SHELX97 read: '
$c2x --cell SiC8.res | sed '/^#/d' > out.cell
pass out.cell SiC8.cell

# Test ability to read cell files

echo -n 'Cell read: '
$c2x --cell SiC8.cell out.cell
pass out.cell SiC8.cell

echo -n 'Cell_abs read: '
$c2x --cell SiC8_abs.cell out.cell
npass out.cell SiC8.cell

echo -n 'Cell_abc read/write: '
$c2x --cell_abc SiC8.cell out.cell
$c2x --cell out.cell out2.cell
npass out2.cell SiC8.cell

echo -n 'Cell_abs read/write (atomic units): '
$c2x --cell_abs -u SiC8.cell out.cell
$c2x --cell out.cell out2.cell
npass out2.cell SiC8.cell

echo -n 'Cell_abc_abs read/write (atomic units): '
$c2x --cell_abc_abs -u SiC8.cell out.cell
$c2x --cell out.cell out2.cell
npass out2.cell SiC8.cell

echo -n 'Cell read with subspecies: '
$c2x --cell_abs Methane.cell out.cell
npass out.cell Methane.cell

echo -n 'Cell read with arithmetic: '
$c2x --cell_abc SiC_4Ha.cell out.cell
npass out.cell SiC_4H.cell

echo -n 'Cell to cif to cell with symmetry: '
$c2x --cif SiC_4H_sym.cell out.cif
$c2x --cell_abc out.cif out.cell
npass out.cell SiC_4H_sym.cell
rm out.cif

echo -n 'Onetep read: '
$c2x -1 --cell SiC8.one out.cell
npass SiC8.cell out.cell

echo -n 'Onetep read with include: '
$c2x -1 --cell SiC8_inc.one out.cell
npass SiC8.cell out.cell

echo -n 'FDF read: '
$c2x --cell FeO_inc.fdf out.cell
npass FeO.cell out.cell
rm out.cell

echo -n 'FDF read(2): '
$c2x --cell FeO_inc2.fdf out.cell
npass FeO.cell out.cell
rm out.cell

echo -n 'FDF read(3): '
$c2x --cell FeO_inc3.fdf out.cell
npass FeO.cell out.cell
rm out.cell

echo -n 'FDF pseudopot charge read: '
$c2x -c --cube FeO_inc.fdf out.cube
npass FeO.cube out.cube
rm out.cube

echo -n 'FDF write: '
$c2x --fdf FeO_inc.fdf out.fdf
$c2x --cell out.fdf out.cell
npass FeO.cell out.cell
rm out.fdf

echo -n 'Crystal read : '
$c2x --cell -n calcite.12 out.cell 2>/dev/null
npass calcite.cell out.cell

echo -n 'Crystal read (2): '
$c2x --cell -n quartz_a.d12 out.cell 2>/dev/null
npass quartz_a.cell out.cell

echo -n 'Abinit read: '
$c2x --cell FeO.in out.cell
npass FeO_kpt.cell out.cell

echo -n 'Abinit read (2): '
$c2x --cell -n calcite.in out.cell
npass calcite.cell out.cell

echo -n 'Abinit read (3): '
$c2x --cell_abc amblygonite.abi out.cell
npass amblygonite.cell out.cell

echo -n 'Abinit read (4): '
$c2x --cell_abc si2.in out.cell
npass si2.cell out.cell

echo -n 'QE read: '
$c2x --cell -Q2 NaCl.in out.cell
npass NaCl.cell out.cell

echo -n 'QE read (2): '
$c2x -X='(5.69,0,0)(0,5.69,0)(0,0,5.69)' -Q2 --cell NaCl_2.in out.cell
# remove k-points
head -17 NaCl.cell > out2.cell
npass out2.cell out.cell

echo -n 'Siesta .XV read/write: '
$c2x --xv FeO.XV out.dat
npass FeO.XV out.dat
rm out.dat

echo -n 'XYZe read/write: '
$c2x --xyze SiC8.cell SiC8.xyz
$c2x --cell SiC8.xyz out.cell
npass SiC8.cell out.cell
rm out.cell SiC8.xyz

echo -n 'Elk read: '
ln -sf GaAs.elk elk.in
$c2x --cell elk.in out.cell
npass out.cell GaAs.cell
rm elk.in out.cell

echo -n 'Elk write: '
$c2x --elk calcite.cell elk.in
$c2x --cell elk.in out.cell
npass out.cell calcite.cell
rm out.cell elk.in

# Python output test

echo -n 'Pya write: '
$c2x --pya SiC8.cell out.pya
npass out.pya SiC8.pya
rm out.pya

echo -n 'npy write: '
$c2x --npy interp.cube out.npy
bpass out.npy interp.npy
rm out.npy

echo -n 'npy Fortran-ordering read: '
$c2x --npy interpf.npy out.npy
bpass out.npy interp.npy
rm out.npy

# Timeseries

echo -n 'Read timeseries from QE xml file, initial step: '
$c2x --qe ethene_relax.in out1.in
$c2x --qe --frame=0 ethene_relax.xml 2>&1 | grep -v conv > out2.in
npass out1.in out2.in
rm out2.in

echo -n 'Read timeseries from QE 7.x xml file, initial step: '
$c2x --qe --frame=0 ethene_relax73.xml 2>&1 | grep -v conv > out2.in
npass out1.in out2.in
rm out1.in out2.in

echo -n 'Read timeseries from QE xml file, final step: '
$c2x --qe ethene_relax.xml out1.in
$c2x --qe --frame=-1 ethene_relax.xml out2.in
npass out1.in out2.in
rm out1.in out2.in

echo -n 'Convert timeseries from QE xml to Castep geom: '
$c2x --geom ethene_relax.xml out1.geom
$c2x --geom out1.geom 2>&1 | tail -n +3 > out2.geom
tail -n +3 ethene_relax.geom > out3.geom
npass out2.geom out3.geom
rm out1.geom out2.geom out3.geom


# Test ability to find primitive cells

echo -n 'Internal primitive cell finder: '
$c2x -P --cell_abc SiC8.cell out.cell 2>/dev/null
npass out.cell SiC2_abc.cell

echo -n 'Manual transformations: '
$c2x -x='(.5,.5,0)(-.5,.5,0)(0,0,1)' --cell SiC8.cell  out.cell
$c2x -X='(4.36,0,0)(0,4.36,0)(0,0,4.36)' --cell out.cell  out2.cell
npass SiC8.cell out2.cell

echo -n "abc re-orientation"
$c2x -a SiC_4H.xsf out.xsf
npass out.xsf SiC_4H_rot.xsf
rm out.xsf

echo -n 'Nanotube rolling: '
$c2x  --cell -y=10,0 graphene.cell out.cell 2>/dev/null
npass tube1.cell out.cell
echo -n 'Nanotube rolling (2): '
$c2x --cell -y=12,6:24 MoS2.XV out.cell 2>/dev/null
npass tube2.cell out.cell


echo -n 'Failure star: '
$c2x -f -M=2,2,2,.5,.5,.5 --null SiC8.cell 2> out.dat
cat > out2.dat <<EOF
First failure star, integral -1.000000
Typical vector: (0.000000,0.000000,8.720000), length**2=76.038400
EOF
npass out.dat out2.dat

echo -n 'Failure star with symops: '
$c2x -f -M=2,2,2 --null SiC_4H_sym.cell 2> out.dat
cat > out2.dat <<EOF
First failure star, integral 1.000000
Typical vector: (-3.082700,5.339393,0.000000), length**2=38.012157
EOF
npass out.dat out2.dat
rm out.dat out2.dat

echo
echo "Cell tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))

passes=0
fails=0

echo -n 'SPGlib international symbol: '
$c2x --int SiC8.cell 2> out.dat
# Line below changed for c2x 2.31
cat > out2.dat <<EOF
Tol=0.0001  International symmetry is F-43m
EOF
pass out.dat out2.dat

echo -n 'SPGlib international symbol (2): '
$c2x --int SiC2_abc.cell 2> out.dat
pass out.dat out2.dat

echo -n 'SPGlib international symbol (range): '
cat > out2.dat <<EOF
Tol=0.0001  International symmetry is C2/m
Tol=0.00926 International symmetry is R-3m
Tol=0.257   International symmetry is Fm-3m
EOF
$c2x -e=0.0001-0.4 --int LaH.in 2>out.dat
pass out.dat out2.dat

echo -n 'SPGlib primitive cell finder: '
$c2x --primitive --cell_abc SiC8.cell out.cell
npass out.cell SiC2_abc.cell

# Skip next test as spglib's output is not stable
if false
then
echo -n 'SPGlib symmetry ops: '
$c2x --list SiC_4H.cell 2> out.dat
cat > out2.dat <<EOF
12 symmetry operations found.
Identity
 6  axis along (-0.000,-0.000, 1.000) through (0.333333,0.333333,0.000000)
 3  axis along ( 0.000,-0.000, 1.000) through (0.333333,0.333333,0.000000)
 2  axis along ( 0.000, 0.000, 1.000) through (0.333333,0.333333,0.000000)
 3  axis along (-0.000, 0.000, 1.000) through (0.666667,0.666667,0.000000)
 6  axis along ( 0.000, 0.000, 1.000) through (0.333333,0.333333,0.000000)
-2  axis along ( 2.000,-1.000, 0.000) through (0.333333,0.333333,0.250000)
-2  axis along ( 1.000, 0.000, 0.000) through (0,0,0)
-2  axis along ( 1.000, 1.000, 0.000) through (0.333333,0.333333,0.250000)
-2  axis along ( 0.000, 1.000, 0.000) through (0.000000,0.500000,0.000000)
-2  axis along (-1.000, 2.000, 0.000) through (0.333333,0.333333,0.250000)
-2  axis along ( 1.000,-1.000, 0.000) through (0.000000,0.500000,0.000000)
EOF
npass out.dat out2.dat
fi

echo -n 'Symmetry ops from castep_bin file and conversion: '
$c2x -v --symmetry --cell SiC_4H.cell 2>/dev/null | grep '^!' | \
    grep -v axis | sort > out.dat
$c2x -v --cell SiC_4H.castep_bin 2>/dev/null | grep '^!' | \
    grep -v axis | sort > out2.dat

pass out.dat out2.dat

    
rm out.dat out2.dat

echo
echo "SPG tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))

passes=0
fails=0

# Read atoms from check files. Use format which will not record
# forces or pseudopotential blocks

echo -n 'Read atoms from check file (v16.11): '
$c2x --shelx Li_16.11.check out1.res
$c2x --shelx Li.cell out2.res
npass out1.res out2.res

echo -n 'Read atoms and vector spins from castep_bin file: '
$c2x --cell Mn3Sn.castep_bin out.cell
npass out.cell Mn3Sn.cell

# Compare charge density to reconstruction from wavefunction
# Remove the different titles
# Note that this example tests weighting of full and empty bands
# and different kpt weights

# The following needs norm conserving pseudopots
echo -n 'Read charge density from check file vs reconstruction from bands: '
$c2x -c Li_16.11.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- Li_16.11.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf

echo -n 'Ditto, gamma-point test: '
$c2x -c diamond6.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- diamond6.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf
rm out1.xsf out2.xsf

# Repeat for 6.1 format check files

if [ -e Li_6.1.check ]
then
    echo -n 'Read atoms from check file (v6.1): '
    $c2x --shelx Li_6.1.check out1.res
    $c2x --shelx Li.cell out2.res
    npass out1.res out2.res

# Remember Castep 6.1 will not do OTF generation of norm conserving pots

    echo -n 'Read charge density from check file (v6.1): '
    $c2x -c Li_6.1.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
    $c2x -BAW -k=- Li_6.1.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
    npass out1.xsf out2.xsf
fi
rm out1.res out2.res out1.xsf out2.xsf

echo -n 'Read charge density from PWscf file vs reconstruction from bands: '
$c2x -c Li.save/data-file-schema.xml | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- Li.save/data-file-schema.xml | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf
rm out1.xsf out2.xsf

echo -n 'Ditto, gamma point: '
$c2x -c diamond6.save/data-file-schema.xml | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- diamond6.save/data-file-schema.xml | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf
rm out1.xsf out2.xsf

echo -n 'Extract single band from PWscf file'
$c2x -B=3 -k=2 -v --null Li.save/data-file-schema.xml 2> out.dat
grep 'min=' out.dat > out2.dat
echo '  min=1.26849e-05 max=0.0911614 sum=319.678 int=1' > out3.dat
npass out2.dat out3.dat
rm out.dat out2.dat out3.dat

# Need to run abinit with spgroup 1 for these
echo -n 'Read charge density from Abinit file vs reconstruction from bands: '
$c2x -c Li_DEN | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- Li_WFK | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf
rm out1.xsf out2.xsf

echo -n 'Ditto, gamma-point test: '
$c2x -c diamond6_DEN | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out1.xsf
$c2x -BAW -k=- diamond6_WFK | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' > out2.xsf
npass out1.xsf out2.xsf
rm out1.xsf out2.xsf

echo -n 'Extract single band from Abinit file'
$c2x -B=3 -k=1 -v --null diamond6_WFK 2> out.dat
grep 'min=' out.dat > out2.dat
echo '  min=2.91062e-08 max=0.12819 sum=843.969 int=1' > out3.dat
npass out2.dat out3.dat
rm out.dat out2.dat out3.dat

echo -n 'Read header from Abinit binary file: '
$c2x --cell_abc Li_DEN out1.cell
head -15 Li.cell > out2.cell
npass out1.cell out2.cell
rm out1.cell out2.cell

echo -n 'Checking VASP writing and re-reading: '

$c2x --vasp -15 -c HCl.check HCl.CHGCAR
$c2x --cube -c HCl.check | sed -e '/^1 /d' -e '/^17 /d' > HCl.cube
$c2x --cube -c HCl.CHGCAR | sed -e '/^1 /d' -e '/^17 /d' > out.cube
npass HCl.cube out.cube
rm HCl.cube out.cube

echo -n 'Checking VASP reading and denfmt writing: '
$c2x --denfmt -c HCl.CHGCAR out1.dat
$c2x --denfmt -c HCl.check out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat HCl.CHGCAR

echo -n 'Checking VASP WAVECAR reading and writing: '
ln -sf diamond6_54.WAVECAR WAVECAR
$c2x --wavecar45 WAVECAR WC45 2>/dev/null
$c2x --gcoeff WAVECAR out1.gc 2>/dev/null
ln -sf WC45 WAVECAR
$c2x --gcoeff WAVECAR out2.gc 2>/dev/null
npass out1.gc out2.gc
rm out1.gc out2.gc WAVECAR WC45

echo -n 'Checking VASP WAVECAR reading and writing (2): '
ln -sf diamond6_54.WAVECAR WAVECAR
$c2x --gcoeff_sorted WAVECAR GCOEFF.txt 2>/dev/null
rm WAVECAR
$c2x --wavecar GCOEFF.txt WAVECAR 
$c2x --gcoeff_sorted WAVECAR out2.gc 2>/dev/null
npass GCOEFF.txt out2.gc
rm GCOEFF.txt out2.gc WAVECAR


echo -n 'Checking cube reading/writing and density interpolation: '

tol='-a 5e-6 -r 3e-5'
$c2x -c -15 -i=64,72,75 --cube HCl.check out.cube
$c2x -c -i=45,40,32 --cube out.cube out2.cube
$c2x -c -i=45,40,32 --cube HCl.check out.cube
npass out.cube out2.cube
rm out.cube out2.cube
tol="$tol1 $tol2"

echo -n 'Further check of interpolation with transforms: '
$c2x -c -x='(1,0,0)(0,2,0)(0,0,3)' HCl.check out.xsf
$c2x -c -Q -x=1x2x3 HCl.check out2.xsf
npass out.xsf out2.xsf
rm out.xsf out2.xsf

echo -n 'Checking line interpolation: '
$c2x -c -P=H1:Cl1:9 HCl.check | sed -e '/^#/d' -e '/^$/d' > out.dat
npass out.dat HCl.dat
echo -n 'Checking line interpolation (2): '
$c2x -c -P='(.0142857,.33333333,.6):(.1957143,.33333333,.6):9' HCl.check | \
    sed -e '/^#/d' -e '/^$/d' > out.dat
npass out.dat HCl.dat
echo -n 'Checking line interpolation (3): '
$c2x -c -P=H:Cl:9 HCl.check | sed -e '/^#/d' -e '/^$/d' > out.dat
npass out.dat HCl.dat
rm out.dat

echo -n 'Checking point interpolation: '
$c2x -c -z=Cl HCl.check | sed 's/^.*: //' > out1.dat
echo 0.35408 > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking point interpolation (2): '
$c2x -c -z=O1 FeO.castep_bin 2>&1 > out1.dat
$c2x -c -z=O2 FeO.castep_bin 2>&1 > out2.dat
x=`cat out1.dat`
x=${x##*:}
[ -n "$x" ] && echo $x > out1.dat
x=`cat out2.dat`
x=${x##*:}
[ -n "$x" ] && echo $x > out2.dat
npass out1.dat out2.dat

rm out1.dat out2.dat

echo -n 'Checking point interpolation (3): '
$c2x -z='(.125,0,0)' interp.cube > out1.dat
x=`cat out1.dat`
x=${x##*:}
[ -n "$x" ] && echo $x > out1.dat
echo 0.25 > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking point interpolation (tricubic): '
$c2x -z='(.125,0,0)' --tricubic interp.cube > out1.dat
x=`cat out1.dat`
x=${x##*:}
[ -n "$x" ] && echo $x > out1.dat
echo 0.15625 > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking grid interpolation: '
$c2x -c -x='(-1,1,0)(-1,0,1)(1,1,1):20x20x45' --npy --fft si2.castep_bin si2_6.npy 2>/dev/null
$c2x --cell -x='(-1,1,0)(-1,0,1)(1,1,1)' si2.castep_bin si2_6.cell 2>/dev/null
$c2x -cn si2.castep_bin 2>/dev/null | sed -e '/^#/d' -e 's/^BEGIN_DATAGRID_3D.*/BEGIN_DATAGRID_3D/' > out1.xsf
$c2x -c --fft --merge -x='(-1/3,-1/3,1/3)(2/3,-1/3,1/3)(-1/3,2/3,1/3):16x16x16' si2_6.cell si2_6.npy  2>/dev/null | sed -e 's/^BEGIN_DATAGRID_3D.*/BEGIN_DATAGRID_3D/' > out2.xsf
npass out1.xsf out2.xsf
rm si2_6.npy si2_6.cell out1.xsf out2.xsf

echo -n 'Checking spherical averaging: '
tol="-r 1e-5 -a 1e-4"
$c2x -P=M:R0:150 -c -v Li_charged.castep_bin /dev/null 2> out.dat
grep "Spherical integral:" out.dat > out1.dat
echo "Spherical integral: 2.0" > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat out.dat

echo -n 'Checking spherical accumulation: '
$c2x -P=M:R0:150a -c  Li_charged.castep_bin out.dat 2>/dev/null
tail -1 out.dat > out1.dat
echo "2.5 2.0" > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat out.dat
tol="$tol1 $tol2"

# This one should be exact, and 4pi/3
echo -n 'Checking spherical integration of constant with --simpson: '
$c2x -v  --merge --simpson -z='(0.5,0.5,0.5):1i' cube.cell const.npy 2>&1 | grep 'Spherical integral' > out1.dat
echo 'Spherical integral: 4.188790' > out2.dat
npass out1.dat out2.dat

# This one should be exact, and 4pi/3
echo -n 'Checking spherical integration of constant with --cubic: '
$c2x -v  --merge --cubic -z='(0.5,0.5,0.5):1i' cube.cell const.npy 2>&1 | grep 'Spherical integral' > out1.dat
npass out1.dat out2.dat

# This one will not be exact
echo -n 'Checking spherical integration of constant (linear) with fffff: '
$c2x -v  --merge -z='(0.5,0.5,0.5):1ifffff' cube.cell const.npy 2>&1 | grep 'Spherical integral' > out1.dat
tol="-r 1e-4 -a 1e-3"
npass out1.dat out2.dat
tol="$tol1 $tol2"

# This will not be exact, analytic result 4pi/5
echo -n 'Checking spherical integration of quadratic with --cubic: '
$c2x -v  --merge --cubic -z='(0.5,0.5,0.5):1ifff' cube.cell quad.npy 2>&1 | grep 'Spherical integral' > out1.dat
echo 'Spherical integral: 2.5132741' > out2.dat
npass out1.dat out2.dat

# This one should be exact, and 4pi
echo -n 'Checking cylindrical integration of constant: '
$c2x --merge --cubic -P='(0.5,0.5,0.5):r1:5a' cube.cell const.npy 2>/dev/null | tail -1 > out1.dat
echo "1 12.5664" > out2.dat
npass out1.dat out2.dat

# This one should be exact, and 2pi
echo -n 'Checking cylindrical integration of quadratic with --cubic: '
$c2x --merge --cubic -P='(0.5,0.5,0.5):r1:5a' cube.cell cylinder.npy 2>/dev/null | tail -1 > out1.dat
echo "1 6.28319" > out2.dat
npass out1.dat out2.dat

echo -n 'Checking spin density extraction and .castep_bin reading: '
$c2x -vs --null FeO.castep_bin 2>&1 | grep 'min=' > out1.dat
echo '  min=-8.45041 max=8.45102 sum=0.119667 int=0.00030401 int|s|=7.0031' > out2.dat
npass out1.dat out2.dat


# Check that the sum of the integrals of two spinor components of
# a single band is one

echo -n 'Checking spinor extraction: '
$c2x -B=1 -S=0 --null -v GaAs.check 2>&1 | grep 'min=' > out1.dat
$c2x -B=1 -S=1 --null -v GaAs.check 2>&1 | grep 'min=' > out2.dat
x=`cat out1.dat`
x=${x##*int=}
y=`cat out2.dat`
y=${y##*int=}
echo $x + $y | bc -q > out1.dat
echo 1 > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking charge and spin density extraction from Siesta .RHO: '
$c2x -cvs --null FeO.RHO 2>&1 | grep 'min=' > out1.dat
cat > out2.dat <<EOF
  min=0.00203035 max=6.76355 sum=5643.1 int=28.0004
  min=-3.55532 max=3.55532 sum=-1.25688e-08 int=-6.2365e-11 int|s|=7.05555
EOF
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking .RHO writing from Siesta: '
$c2x -cs --xv FeO.RHO out.XV 2>/dev/null
$c2x -s --cube FeO.RHO out.cube 2>/dev/null
$c2x -s --cube out.RHO out2.cube 2>/dev/null
tol="-r 1e-5 -a 2e-5"
npass out.cube out2.cube
tol="$tol1 $tol2"
rm out2.cube out.cube out.RHO out.XV

echo -n 'Checking charge density reading from Siesta vector spin RHO: '
$c2x -cv --null fe_noncol_kp.RHO 2>&1 | grep 'min=' > out1.dat
cat > out2.dat <<EOF
  min=0 max=5.91203 sum=6910.34 int=24.0002
EOF
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking Siesta vector spin extraction with spherical integration: '
$c2x -s --vec2force=0.96i fe_noncol_kp.RHO 2>&1 | tail -3 > out1.dat
cat > out2.dat <<EOF
26   0.0000000   0.0000000   1.9283218   0.0626296   0.0000000   2.7095781
26   0.0000000   0.0000000   0.0000000   1.1866760  -0.0000000   0.0000013
26   0.0000000   0.0000000   6.5385136   0.0626293  -0.0000000  -2.7095775
EOF
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking Castep vector spin extraction with spherical integration: '
$c2x -s --vec2force=1.35i  Mn3Sn.castep_bin 2>&1 | tail -8 > out1.dat
cat > out2.dat <<EOF
25   2.7897534   3.2474496   1.1325000   3.0542672  -0.1928168  -0.4512538
25   0.0452523   3.2474496   1.1325000  -1.6312414  -2.6138573   0.2760893
25   1.4175028   0.8706419   1.1325000  -1.5004415   2.6783969   0.3852940
25   1.4174972   4.0397221   3.3975000  -1.4098802   2.5029022  -1.1362228
25   2.7897477   1.6629144   3.3975000  -1.5976455  -2.6474015  -0.0537890
25   0.0452466   1.6629144   3.3975000   3.0423194  -0.0967655   0.5769532
50   4.2524972   0.8183956   1.1325000   0.0010251   0.0027278   0.0034250
50  -1.4174972   4.0919684   3.3975000   0.0009688   0.0025619   0.0021751
EOF
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking charged cell energy correction (0D, cubic): '
$c2x -q --null Li_charged.castep_bin 2>&1 | grep energy: > out1.dat
cat > out2.dat <<EOF
Original energy:             -196.510105 eV
Corrected energy:            -192.424497 eV
Final corrected energy:      -192.360264 eV
EOF
npass out1.dat out2.dat
rm out1.dat out2.dat

echo -n 'Checking charged cell energy correction (2D): '
$c2x -qc --null graphene_charged.castep_bin 2>&1 | grep energy: > out1.dat
cat > out2.dat <<EOF
Original energy:          -1268.642479 eV
Corrected energy:         -1360.881268 eV
Final corrected energy:   -1419.994945 eV
EOF
npass out1.dat out2.dat
rm out1.dat out2.dat

echo
echo "Density tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))

passes=0
fails=0
echo -n 'Checking bands writing: '
$c2x --bands Al.bands out.bands 2>/dev/null
npass Al.bands out.bands
rm out.bands
echo -n 'Checking BXSF writing: '
$c2x --bxsf Al.bands out.bxsf 2>/dev/null
npass Al.bxsf out.bxsf
rm out.bxsf
echo -n 'Checking band parity (Castep): '
$c2x -I --null ethene_gamma.check 2> out.log
npass ethene_gamma.parity out.log
rm out.log
echo -n 'Checking band parity (VASP): '
$c2x --wavecar ethene_gamma.check WAVECAR
$c2x -I --null WAVECAR 2>&1 | grep -v WAVECAR > out.log
npass ethene_gamma.parity out.log
rm WAVECAR out.log
echo -n 'Checking band gap (from Castep .bands): '
$c2x --gap --null SiC_4H.bands 2>&1 | grep eV > out1.dat
cat > out2.dat <<EOF
Valence band maximum:    6.856599 eV at k=(0.125000,0.125000,0.000000)
Conduction band minimum: 9.582341 eV at k=(0.375000,0.375000,0.000000)
Direct gap:              3.543038 eV at k=(0.375000,0.375000,0.000000)
Indirect gap:            2.725742 eV
EOF
npass out1.dat out2.dat
rm out1.dat out2.dat
echo -n 'Checking band gap (from Abinit EIG): '
$c2x --gap --null SiC_4H_EIG 2>&1 | grep eV > out1.dat
cat > out2.dat <<EOF
Valence band maximum:    7.011558 eV at k=(0.125000,0.125000,0.000000)
Conduction band minimum: 9.729975 eV at k=(0.375000,0.375000,0.000000)
Direct gap:              3.556528 eV at k=(0.375000,0.375000,0.000000)
Indirect gap:            2.718417 eV
EOF
npass out1.dat out2.dat
rm out1.dat out2.dat

echo
echo "Bands tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))

passes=0
fails=0
echo -n 'Checking merging: '
$c2x -15 --cell SiC_4H.castep_bin out.cell
$c2x -15 --npy -c SiC_4H.castep_bin out.npy
$c2x --merge out.cell out.npy - | sed 's/(null)/Density/' > out.xsf
$c2x -c -n SiC_4H.castep_bin - | grep -v '^#' > out2.xsf
npass out.xsf out2.xsf
rm out.cell out.npy out.xsf out2.xsf

echo -n 'Checking merging of forces: '
$c2x -s --vec2force=Mn Mn3Sn.castep_bin out1.xsf
$c2x -s --vec2force=Sn Mn3Sn.castep_bin out2.xsf
$c2x -s --vec2force Mn3Sn.castep_bin | grep -v '^#' > out3.xsf
$c2x --merge out1.xsf out2.xsf out4.xsf 2>/dev/null
npass out3.xsf out4.xsf
rm out1.xsf out2.xsf out3.xsf out4.xsf

echo -n 'Checking differencing: '
$c2x -15 --npy -c SiC_4H.castep_bin out.npy
$c2x --null --diff -cv SiC_4H.castep_bin out.npy 2>&1 | grep min= | sed 's/=/ /g'> out1.dat
echo '  min 0 max 0 sum 0 int 0' > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat out.npy

echo -n 'Checking scaling and adding: '
$c2x -c --npy -R=-2x -15 SiC_4H.castep_bin out.npy
$c2x -c --cube -15 SiC_4H.castep_bin out.cube
$c2x -cv --null --add SiC_4H.castep_bin out.npy out.cube 2>&1 | grep min= | sed 's/=/ /g' > out1.dat
echo '  min 0 max 0 sum 0 int 0' > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat out.npy out.cube

echo -n 'Checking masking: '
gunzip -k mask.npy.gz
$c2x -vc --null --mask diamond6.check mask.npy 2>&1 | grep int= | sed 's/.*int=//' > out1.dat
echo 8 > out2.dat
npass out1.dat out2.dat
rm out1.dat out2.dat mask.npy

echo -n 'Checking grad2 and div: '
$c2x -c --grad2 diamond6.check | sed -e '/BEGIN_DATAGRID_3D/d' -e '/^#/d' >  out1.xsf 
$c2x -c --grad --npy -15 diamond6.check out1.npy
$c2x --merge --div diamond6.check out1.npy | sed -e '/BEGIN_DATAGRID_3D/d' > out2.xsf
npass out1.xsf out2.xsf
rm out1.npy out1.xsf out2.xsf

echo
echo "Combining tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))

passes=0
fails=0
echo 'Testing FBZ code'
for t in lattices/*.cell
do
    echo -n "Testing $t "
    $c2x --fbz --null -v $t 2>&1 | tail -1 > out.dat
    cut -d ' ' -f 3 < out.dat > out1.dat
    cut -d ' ' -f 5 < out.dat > out2.dat
    npass out1.dat out2.dat
done

# Hard to check IBZs as IBZ not unique, and ordering of vertices
# also not unique. So just use volume
echo
echo 'Testing IBZ code (all tests will fail unless spglib present)'
for t in lattices/*.cell
do
    echo -n "Testing $t "
    $c2x --ibz --sym --null -v $t 2>&1 | tail -1 > out.dat
    cut -d ' ' -f 5 < out.dat | tr -d , > out1.dat
    cut -d ' ' -f 8 < out.dat > out2.dat
    npass out1.dat out2.dat
done
rm out.dat out1.dat out2.dat

echo
echo "FBZ/IBZ tests complete."
echo "Passed: $passes"
echo "Failed: $fails"
echo

total_passes=$((total_passes+passes))
total_fails=$((total_fails+fails))


echo "Total score: passed $total_passes"
echo "             failed $total_fails"
